@@ -103,10 +103,10 @@ Best run this once to get a feel for the platforms and devices in your
 system.
 
    for my $platform (OpenCL::platforms) {
-      printf "platform: %s\n", $platform->info (OpenCL::PLATFORM_NAME);
-      printf "extensions: %s\n", $platform->info (OpenCL::PLATFORM_EXTENSIONS);
+      printf "platform: %s\n", $platform->name;
+      printf "extensions: %s\n", $platform->extensions;
       for my $device ($platform->devices) {
-         printf "+ device: %s\n", $device->info (OpenCL::DEVICE_NAME);
+         printf "+ device: %s\n", $device->name;
          my $ctx = $device->context;
          # do stuff
       }
@@ -163,7 +163,7 @@ functions.
 
    # build croaks on compile errors, so catch it and print the compile errors
    eval { $prog->build ($dev); 1 }
-      or die $prog->build_info ($dev, OpenCL::PROGRAM_BUILD_LOG);
+      or die $prog->build_log;
 
    my $kernel = $prog->kernel ("squareit");
 
@@ -324,7 +324,7 @@ Returns a list of matching OpenCL::Device objects.
 
 =item $ctx = $platform->context_from_type ($properties, $type = OpenCL::DEVICE_TYPE_DEFAULT, $notify = undef)
 
-Tries to create a context. Never worked for me, and you need devices explitly anyway.
+Tries to create a context. Never worked for me, and you need devices explicitly anyway.
 
 L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clCreateContextFromType.html>
 
@@ -338,36 +338,35 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clCreateContext.html
 =item $packed_value = $platform->info ($name)
 
 Calls C<clGetPlatformInfo> and returns the packed, raw value - for
-strings, this will be the string, for other values you probably need to
-use the correct C<unpack>.
+strings, this will be the string (possibly including terminating \0), for
+other values you probably need to use the correct C<unpack>.
 
-It's best to avoid this method and use one of the predefined C<get_*>
-methods.
+It's best to avoid this method and use one of the following convenience
+wrappers.
 
 L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetPlatformInfo.html>
 
 =for gengetinfo begin platform
 
-
 =item $string = $platform->profile
 
-Calls C<clGetPlatformInfo> with C<CL_PLATFORM_PROFILE> and returns the result(s).
+Calls C<clGetPlatformInfo> with C<CL_PLATFORM_PROFILE> and returns the result.
 
 =item $string = $platform->version
 
-Calls C<clGetPlatformInfo> with C<CL_PLATFORM_VERSION> and returns the result(s).
+Calls C<clGetPlatformInfo> with C<CL_PLATFORM_VERSION> and returns the result.
 
 =item $string = $platform->name
 
-Calls C<clGetPlatformInfo> with C<CL_PLATFORM_NAME> and returns the result(s).
+Calls C<clGetPlatformInfo> with C<CL_PLATFORM_NAME> and returns the result.
 
 =item $string = $platform->vendor
 
-Calls C<clGetPlatformInfo> with C<CL_PLATFORM_VENDOR> and returns the result(s).
+Calls C<clGetPlatformInfo> with C<CL_PLATFORM_VENDOR> and returns the result.
 
 =item $string = $platform->extensions
 
-Calls C<clGetPlatformInfo> with C<CL_PLATFORM_EXTENSIONS> and returns the result(s).
+Calls C<clGetPlatformInfo> with C<CL_PLATFORM_EXTENSIONS> and returns the result.
 
 =for gengetinfo end platform
 
@@ -385,270 +384,269 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetDeviceInfo.html
 
 =for gengetinfo begin device
 
-
 =item $device_type = $device->type
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_TYPE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_TYPE> and returns the result.
 
 =item $uint = $device->vendor_id
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_VENDOR_ID> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_VENDOR_ID> and returns the result.
 
 =item $uint = $device->max_compute_units
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_COMPUTE_UNITS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_COMPUTE_UNITS> and returns the result.
 
 =item $uint = $device->max_work_item_dimensions
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS> and returns the result.
 
 =item $int = $device->max_work_group_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_GROUP_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_GROUP_SIZE> and returns the result.
 
 =item @ints = $device->max_work_item_sizes
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_ITEM_SIZES> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WORK_ITEM_SIZES> and returns the result.
 
 =item $uint = $device->preferred_vector_width_char
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_CHAR> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_CHAR> and returns the result.
 
 =item $uint = $device->preferred_vector_width_short
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_SHORT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_SHORT> and returns the result.
 
 =item $uint = $device->preferred_vector_width_int
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_INT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_INT> and returns the result.
 
 =item $uint = $device->preferred_vector_width_long
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_LONG> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_LONG> and returns the result.
 
 =item $uint = $device->preferred_vector_width_float
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_FLOAT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_FLOAT> and returns the result.
 
 =item $uint = $device->preferred_vector_width_double
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_DOUBLE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_DOUBLE> and returns the result.
 
 =item $uint = $device->max_clock_frequency
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CLOCK_FREQUENCY> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CLOCK_FREQUENCY> and returns the result.
 
 =item $bitfield = $device->address_bits
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_ADDRESS_BITS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_ADDRESS_BITS> and returns the result.
 
 =item $uint = $device->max_read_image_args
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_READ_IMAGE_ARGS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_READ_IMAGE_ARGS> and returns the result.
 
 =item $uint = $device->max_write_image_args
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WRITE_IMAGE_ARGS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_WRITE_IMAGE_ARGS> and returns the result.
 
 =item $ulong = $device->max_mem_alloc_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_MEM_ALLOC_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_MEM_ALLOC_SIZE> and returns the result.
 
 =item $int = $device->image2d_max_width
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE2D_MAX_WIDTH> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE2D_MAX_WIDTH> and returns the result.
 
 =item $int = $device->image2d_max_height
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE2D_MAX_HEIGHT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE2D_MAX_HEIGHT> and returns the result.
 
 =item $int = $device->image3d_max_width
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_WIDTH> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_WIDTH> and returns the result.
 
 =item $int = $device->image3d_max_height
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_HEIGHT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_HEIGHT> and returns the result.
 
 =item $int = $device->image3d_max_depth
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_DEPTH> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE3D_MAX_DEPTH> and returns the result.
 
 =item $uint = $device->image_support
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE_SUPPORT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_IMAGE_SUPPORT> and returns the result.
 
 =item $int = $device->max_parameter_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_PARAMETER_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_PARAMETER_SIZE> and returns the result.
 
 =item $uint = $device->max_samplers
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_SAMPLERS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_SAMPLERS> and returns the result.
 
 =item $uint = $device->mem_base_addr_align
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MEM_BASE_ADDR_ALIGN> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MEM_BASE_ADDR_ALIGN> and returns the result.
 
 =item $uint = $device->min_data_type_align_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MIN_DATA_TYPE_ALIGN_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MIN_DATA_TYPE_ALIGN_SIZE> and returns the result.
 
 =item $device_fp_config = $device->single_fp_config
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_SINGLE_FP_CONFIG> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_SINGLE_FP_CONFIG> and returns the result.
 
 =item $device_mem_cache_type = $device->global_mem_cache_type
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHE_TYPE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHE_TYPE> and returns the result.
 
 =item $uint = $device->global_mem_cacheline_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHELINE_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHELINE_SIZE> and returns the result.
 
 =item $ulong = $device->global_mem_cache_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHE_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_CACHE_SIZE> and returns the result.
 
 =item $ulong = $device->global_mem_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_GLOBAL_MEM_SIZE> and returns the result.
 
 =item $ulong = $device->max_constant_buffer_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE> and returns the result.
 
 =item $uint = $device->max_constant_args
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CONSTANT_ARGS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_MAX_CONSTANT_ARGS> and returns the result.
 
 =item $device_local_mem_type = $device->local_mem_type
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_LOCAL_MEM_TYPE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_LOCAL_MEM_TYPE> and returns the result.
 
 =item $ulong = $device->local_mem_size
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_LOCAL_MEM_SIZE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_LOCAL_MEM_SIZE> and returns the result.
 
 =item $boolean = $device->error_correction_support
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_ERROR_CORRECTION_SUPPORT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_ERROR_CORRECTION_SUPPORT> and returns the result.
 
 =item $int = $device->profiling_timer_resolution
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PROFILING_TIMER_RESOLUTION> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PROFILING_TIMER_RESOLUTION> and returns the result.
 
 =item $boolean = $device->endian_little
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_ENDIAN_LITTLE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_ENDIAN_LITTLE> and returns the result.
 
 =item $boolean = $device->available
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_AVAILABLE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_AVAILABLE> and returns the result.
 
 =item $boolean = $device->compiler_available
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_COMPILER_AVAILABLE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_COMPILER_AVAILABLE> and returns the result.
 
 =item $device_exec_capabilities = $device->execution_capabilities
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_EXECUTION_CAPABILITIES> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_EXECUTION_CAPABILITIES> and returns the result.
 
 =item $command_queue_properties = $device->properties
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_QUEUE_PROPERTIES> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_QUEUE_PROPERTIES> and returns the result.
 
 =item $ = $device->platform
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PLATFORM> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PLATFORM> and returns the result.
 
 =item $string = $device->name
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NAME> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NAME> and returns the result.
 
 =item $string = $device->vendor
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_VENDOR> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_VENDOR> and returns the result.
 
 =item $string = $device->driver_version
 
-Calls C<clGetDeviceInfo> with C<CL_DRIVER_VERSION> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DRIVER_VERSION> and returns the result.
 
 =item $string = $device->profile
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PROFILE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PROFILE> and returns the result.
 
 =item $string = $device->version
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_VERSION> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_VERSION> and returns the result.
 
 =item $string = $device->extensions
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_EXTENSIONS> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_EXTENSIONS> and returns the result.
 
 =item $uint = $device->preferred_vector_width_half
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_HALF> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PREFERRED_VECTOR_WIDTH_HALF> and returns the result.
 
 =item $uint = $device->native_vector_width_char
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_CHAR> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_CHAR> and returns the result.
 
 =item $uint = $device->native_vector_width_short
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_SHORT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_SHORT> and returns the result.
 
 =item $uint = $device->native_vector_width_int
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_INT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_INT> and returns the result.
 
 =item $uint = $device->native_vector_width_long
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_LONG> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_LONG> and returns the result.
 
 =item $uint = $device->native_vector_width_float
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_FLOAT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_FLOAT> and returns the result.
 
 =item $uint = $device->native_vector_width_double
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_DOUBLE> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_DOUBLE> and returns the result.
 
 =item $uint = $device->native_vector_width_half
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF> and returns the result.
 
 =item $device_fp_config = $device->double_fp_config
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_DOUBLE_FP_CONFIG> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_DOUBLE_FP_CONFIG> and returns the result.
 
 =item $device_fp_config = $device->half_fp_config
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_HALF_FP_CONFIG> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_HALF_FP_CONFIG> and returns the result.
 
 =item $boolean = $device->host_unified_memory
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_HOST_UNIFIED_MEMORY> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_HOST_UNIFIED_MEMORY> and returns the result.
 
 =item $device = $device->parent_device_ext
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARENT_DEVICE_EXT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARENT_DEVICE_EXT> and returns the result.
 
 =item @device_partition_property_exts = $device->partition_types_ext
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARTITION_TYPES_EXT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARTITION_TYPES_EXT> and returns the result.
 
 =item @device_partition_property_exts = $device->affinity_domains_ext
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_AFFINITY_DOMAINS_EXT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_AFFINITY_DOMAINS_EXT> and returns the result.
 
 =item $uint = $device->reference_count_ext 
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_REFERENCE_COUNT_EXT > and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_REFERENCE_COUNT_EXT > and returns the result.
 
 =item @device_partition_property_exts = $device->partition_style_ext
 
-Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARTITION_STYLE_EXT> and returns the result(s).
+Calls C<clGetDeviceInfo> with C<CL_DEVICE_PARTITION_STYLE_EXT> and returns the result.
 
 =for gengetinfo end device
 
@@ -719,22 +717,21 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetContextInfo.htm
 
 =for gengetinfo begin context
 
-
 =item $uint = $context->reference_count
 
-Calls C<clGetContextInfo> with C<CL_CONTEXT_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetContextInfo> with C<CL_CONTEXT_REFERENCE_COUNT> and returns the result.
 
 =item @devices = $context->devices
 
-Calls C<clGetContextInfo> with C<CL_CONTEXT_DEVICES> and returns the result(s).
+Calls C<clGetContextInfo> with C<CL_CONTEXT_DEVICES> and returns the result.
 
 =item @property_ints = $context->properties
 
-Calls C<clGetContextInfo> with C<CL_CONTEXT_PROPERTIES> and returns the result(s).
+Calls C<clGetContextInfo> with C<CL_CONTEXT_PROPERTIES> and returns the result.
 
 =item $uint = $context->num_devices
 
-Calls C<clGetContextInfo> with C<CL_CONTEXT_NUM_DEVICES> and returns the result(s).
+Calls C<clGetContextInfo> with C<CL_CONTEXT_NUM_DEVICES> and returns the result.
 
 =for gengetinfo end context
 
@@ -776,6 +773,14 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clEnqueueWriteBuffer
 
 L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clEnqueueCopyBuffer.html>
 
+=item $ev = $queue->enqueue_read_buffer_rect (OpenCL::Memory buf, cl_bool blocking, $buf_x, $buf_y, $buf_z, $host_x, $host_y, $host_z, $width, $height, $depth, $buf_row_pitch, $buf_slice_pitch, $host_row_pitch, $host_slice_pitch, $data, $wait_events...)
+
+http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clEnqueueReadBufferRect.html
+
+=item $ev = $queue->enqueue_write_buffer_rect (OpenCL::Memory buf, cl_bool blocking, $buf_x, $buf_y, $buf_z, $host_x, $host_y, $host_z, $width, $height, $depth, $buf_row_pitch, $buf_slice_pitch, $host_row_pitch, $host_slice_pitch, $data, $wait_events...)
+
+http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clEnqueueWriteBufferRect.html
+
 =item $ev = $queue->enqueue_read_image ($src, $blocking, $x, $y, $z, $width, $height, $depth, $row_pitch, $slice_pitch, $data, $wait_events...)
 
 L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clEnqueueReadImage.html>
@@ -852,22 +857,21 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetCommandQueueInf
 
 =for gengetinfo begin command_queue
 
-
 =item $ctx = $command_queue->context
 
-Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_CONTEXT> and returns the result(s).
+Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_CONTEXT> and returns the result.
 
 =item $device = $command_queue->device
 
-Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_DEVICE> and returns the result(s).
+Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_DEVICE> and returns the result.
 
 =item $uint = $command_queue->reference_count
 
-Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_REFERENCE_COUNT> and returns the result.
 
 =item $command_queue_properties = $command_queue->properties
 
-Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_PROPERTIES> and returns the result(s).
+Calls C<clGetCommandQueueInfo> with C<CL_QUEUE_PROPERTIES> and returns the result.
 
 =for gengetinfo end command_queue
 
@@ -888,42 +892,41 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetMemObjectInfo.h
 
 =for gengetinfo begin mem
 
-
 =item $mem_object_type = $mem->type
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_TYPE> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_TYPE> and returns the result.
 
 =item $mem_flags = $mem->flags
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_FLAGS> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_FLAGS> and returns the result.
 
 =item $int = $mem->size
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_SIZE> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_SIZE> and returns the result.
 
 =item $ptr_value = $mem->host_ptr
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_HOST_PTR> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_HOST_PTR> and returns the result.
 
 =item $uint = $mem->map_count
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_MAP_COUNT> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_MAP_COUNT> and returns the result.
 
 =item $uint = $mem->reference_count
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_REFERENCE_COUNT> and returns the result.
 
 =item $ctx = $mem->context
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_CONTEXT> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_CONTEXT> and returns the result.
 
 =item $mem = $mem->associated_memobject
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_ASSOCIATED_MEMOBJECT> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_ASSOCIATED_MEMOBJECT> and returns the result.
 
 =item $int = $mem->offset
 
-Calls C<clGetMemObjectInfo> with C<CL_MEM_OFFSET> and returns the result(s).
+Calls C<clGetMemObjectInfo> with C<CL_MEM_OFFSET> and returns the result.
 
 =for gengetinfo end mem
 
@@ -946,30 +949,29 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetImageInfo.html>
 
 =for gengetinfo begin image
 
-
 =item $int = $image->element_size
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_ELEMENT_SIZE> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_ELEMENT_SIZE> and returns the result.
 
 =item $int = $image->row_pitch
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_ROW_PITCH> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_ROW_PITCH> and returns the result.
 
 =item $int = $image->slice_pitch
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_SLICE_PITCH> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_SLICE_PITCH> and returns the result.
 
 =item $int = $image->width
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_WIDTH> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_WIDTH> and returns the result.
 
 =item $int = $image->height
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_HEIGHT> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_HEIGHT> and returns the result.
 
 =item $int = $image->depth
 
-Calls C<clGetImageInfo> with C<CL_IMAGE_DEPTH> and returns the result(s).
+Calls C<clGetImageInfo> with C<CL_IMAGE_DEPTH> and returns the result.
 
 =for gengetinfo end image
 
@@ -987,26 +989,25 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetSamplerInfo.htm
 
 =for gengetinfo begin sampler
 
-
 =item $uint = $sampler->reference_count
 
-Calls C<clGetSamplerInfo> with C<CL_SAMPLER_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetSamplerInfo> with C<CL_SAMPLER_REFERENCE_COUNT> and returns the result.
 
 =item $ctx = $sampler->context
 
-Calls C<clGetSamplerInfo> with C<CL_SAMPLER_CONTEXT> and returns the result(s).
+Calls C<clGetSamplerInfo> with C<CL_SAMPLER_CONTEXT> and returns the result.
 
 =item $addressing_mode = $sampler->normalized_coords
 
-Calls C<clGetSamplerInfo> with C<CL_SAMPLER_NORMALIZED_COORDS> and returns the result(s).
+Calls C<clGetSamplerInfo> with C<CL_SAMPLER_NORMALIZED_COORDS> and returns the result.
 
 =item $filter_mode = $sampler->addressing_mode
 
-Calls C<clGetSamplerInfo> with C<CL_SAMPLER_ADDRESSING_MODE> and returns the result(s).
+Calls C<clGetSamplerInfo> with C<CL_SAMPLER_ADDRESSING_MODE> and returns the result.
 
 =item $boolean = $sampler->filter_mode
 
-Calls C<clGetSamplerInfo> with C<CL_SAMPLER_FILTER_MODE> and returns the result(s).
+Calls C<clGetSamplerInfo> with C<CL_SAMPLER_FILTER_MODE> and returns the result.
 
 =for gengetinfo end sampler
 
@@ -1038,18 +1039,17 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clCreateKernel.html>
 
 =for gengetinfo begin program_build
 
-
 =item $build_status = $program->build_status ($device)
 
-Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_STATUS> and returns the result(s).
+Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_STATUS> and returns the result.
 
 =item $string = $program->build_options ($device)
 
-Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_OPTIONS> and returns the result(s).
+Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_OPTIONS> and returns the result.
 
 =item $string = $program->build_log ($device)
 
-Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_LOG> and returns the result(s).
+Calls C<clGetProgramBuildInfo> with C<CL_PROGRAM_BUILD_LOG> and returns the result.
 
 =for gengetinfo end program_build
 
@@ -1061,33 +1061,43 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetProgramInfo.htm
 
 =for gengetinfo begin program
 
-
 =item $uint = $program->reference_count
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_REFERENCE_COUNT> and returns the result.
 
 =item $ctx = $program->context
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_CONTEXT> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_CONTEXT> and returns the result.
 
 =item $uint = $program->num_devices
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_NUM_DEVICES> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_NUM_DEVICES> and returns the result.
 
 =item @devices = $program->devices
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_DEVICES> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_DEVICES> and returns the result.
 
 =item $string = $program->source
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_SOURCE> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_SOURCE> and returns the result.
 
 =item @ints = $program->binary_sizes
 
-Calls C<clGetProgramInfo> with C<CL_PROGRAM_BINARY_SIZES> and returns the result(s).
+Calls C<clGetProgramInfo> with C<CL_PROGRAM_BINARY_SIZES> and returns the result.
 
 =for gengetinfo end program
 
+=item @blobs = $program->binaries
+
+Returns a string for the compiled binary for every device associated with
+the program, empty strings indicate missing programs, and an empty result
+means no program binaries are available.
+
+These "binaries" are often, in fact, informative low-level assembly
+sources.
+
+L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetProgramInfo.html>
+
 =back
 
 =head2 THE OpenCL::Kernel CLASS
@@ -1102,26 +1112,25 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetKernelInfo.html
 
 =for gengetinfo begin kernel
 
-
 =item $string = $kernel->function_name
 
-Calls C<clGetKernelInfo> with C<CL_KERNEL_FUNCTION_NAME> and returns the result(s).
+Calls C<clGetKernelInfo> with C<CL_KERNEL_FUNCTION_NAME> and returns the result.
 
 =item $uint = $kernel->num_args
 
-Calls C<clGetKernelInfo> with C<CL_KERNEL_NUM_ARGS> and returns the result(s).
+Calls C<clGetKernelInfo> with C<CL_KERNEL_NUM_ARGS> and returns the result.
 
 =item $uint = $kernel->reference_count
 
-Calls C<clGetKernelInfo> with C<CL_KERNEL_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetKernelInfo> with C<CL_KERNEL_REFERENCE_COUNT> and returns the result.
 
 =item $ctx = $kernel->context
 
-Calls C<clGetKernelInfo> with C<CL_KERNEL_CONTEXT> and returns the result(s).
+Calls C<clGetKernelInfo> with C<CL_KERNEL_CONTEXT> and returns the result.
 
 =item $program = $kernel->program
 
-Calls C<clGetKernelInfo> with C<CL_KERNEL_PROGRAM> and returns the result(s).
+Calls C<clGetKernelInfo> with C<CL_KERNEL_PROGRAM> and returns the result.
 
 =for gengetinfo end kernel
 
@@ -1136,26 +1145,25 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetKernelWorkGroup
 
 =for gengetinfo begin kernel_work_group
 
-
 =item $int = $kernel->work_group_size ($device)
 
-Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_WORK_GROUP_SIZE> and returns the result(s).
+Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_WORK_GROUP_SIZE> and returns the result.
 
 =item @ints = $kernel->compile_work_group_size ($device)
 
-Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_COMPILE_WORK_GROUP_SIZE> and returns the result(s).
+Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_COMPILE_WORK_GROUP_SIZE> and returns the result.
 
 =item $ulong = $kernel->local_mem_size ($device)
 
-Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_LOCAL_MEM_SIZE> and returns the result(s).
+Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_LOCAL_MEM_SIZE> and returns the result.
 
 =item $int = $kernel->preferred_work_group_size_multiple ($device)
 
-Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE> and returns the result(s).
+Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE> and returns the result.
 
 =item $ulong = $kernel->private_mem_size ($device)
 
-Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_PRIVATE_MEM_SIZE> and returns the result(s).
+Calls C<clGetKernelWorkGroupInfo> with C<CL_KERNEL_PRIVATE_MEM_SIZE> and returns the result.
 
 =for gengetinfo end kernel_work_group
 
@@ -1197,26 +1205,25 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetEventInfo.html>
 
 =for gengetinfo begin event
 
-
 =item $queue = $event->command_queue
 
-Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_QUEUE> and returns the result(s).
+Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_QUEUE> and returns the result.
 
 =item $command_type = $event->command_type
 
-Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_TYPE> and returns the result(s).
+Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_TYPE> and returns the result.
 
 =item $uint = $event->reference_count
 
-Calls C<clGetEventInfo> with C<CL_EVENT_REFERENCE_COUNT> and returns the result(s).
+Calls C<clGetEventInfo> with C<CL_EVENT_REFERENCE_COUNT> and returns the result.
 
 =item $uint = $event->command_execution_status
 
-Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_EXECUTION_STATUS> and returns the result(s).
+Calls C<clGetEventInfo> with C<CL_EVENT_COMMAND_EXECUTION_STATUS> and returns the result.
 
 =item $ctx = $event->context
 
-Calls C<clGetEventInfo> with C<CL_EVENT_CONTEXT> and returns the result(s).
+Calls C<clGetEventInfo> with C<CL_EVENT_CONTEXT> and returns the result.
 
 =for gengetinfo end event
 
@@ -1231,22 +1238,21 @@ L<http://www.khronos.org/registry/cl/sdk/1.1/docs/man/xhtml/clGetProfilingInfo.h
 
 =for gengetinfo begin profiling
 
-
 =item $ulong = $event->profiling_command_queued
 
-Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_QUEUED> and returns the result(s).
+Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_QUEUED> and returns the result.
 
 =item $ulong = $event->profiling_command_submit
 
-Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_SUBMIT> and returns the result(s).
+Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_SUBMIT> and returns the result.
 
 =item $ulong = $event->profiling_command_start
 
-Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_START> and returns the result(s).
+Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_START> and returns the result.
 
 =item $ulong = $event->profiling_command_end
 
-Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_END> and returns the result(s).
+Calls C<clGetEventProfilingInfo> with C<CL_PROFILING_COMMAND_END> and returns the result.
 
 =for gengetinfo end profiling
 
@@ -1271,7 +1277,7 @@ package OpenCL;
 use common::sense;
 
 BEGIN {
-   our $VERSION = '0.55';
+   our $VERSION = '0.59';
 
    require XSLoader;
    XSLoader::load (__PACKAGE__, $VERSION);
